'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _rsocketFlowable = require('rsocket-flowable');

var _rsocketRpcFrames = require('rsocket-rpc-frames');

var _QueuingFlowableProcessor = require('./QueuingFlowableProcessor');

var _QueuingFlowableProcessor2 = _interopRequireDefault(_QueuingFlowableProcessor);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } } /**
                                                                                                                                                           * Copyright (c) 2017-present, Netifi Inc.
                                                                                                                                                           *
                                                                                                                                                           * Licensed under the Apache License, Version 2.0 (the "License");
                                                                                                                                                           * you may not use this file except in compliance with the License.
                                                                                                                                                           * You may obtain a copy of the License at
                                                                                                                                                           *
                                                                                                                                                           *       http://www.apache.org/licenses/LICENSE-2.0
                                                                                                                                                           *
                                                                                                                                                           * Unless required by applicable law or agreed to in writing, software
                                                                                                                                                           * distributed under the License is distributed on an "AS IS" BASIS,
                                                                                                                                                           * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
                                                                                                                                                           * See the License for the specific language governing permissions and
                                                                                                                                                           * limitations under the License.
                                                                                                                                                           *
                                                                                                                                                           * 
                                                                                                                                                           */

var RequestHandlingRSocket = function () {
  function RequestHandlingRSocket() {
    _classCallCheck(this, RequestHandlingRSocket);

    this._registeredServices = new Map();
  }

  RequestHandlingRSocket.prototype.addService = function addService(service, handler) {
    this._registeredServices.set(service, handler);
  };

  RequestHandlingRSocket.prototype.fireAndForget = function fireAndForget(payload) {
    if (payload.metadata == null) {
      throw new Error('metadata is empty');
    }

    var service = (0, _rsocketRpcFrames.getService)(payload.metadata);
    var handler = this._registeredServices.get(service);

    if (handler == null) {
      throw new Error('can not find service ' + service);
    }

    handler.fireAndForget(payload);
  };

  RequestHandlingRSocket.prototype.requestResponse = function requestResponse(payload) {
    try {
      if (payload.metadata == null) {
        return _rsocketFlowable.Single.error(new Error('metadata is empty'));
      }

      var service = (0, _rsocketRpcFrames.getService)(payload.metadata);
      var handler = this._registeredServices.get(service);

      if (handler == null) {
        return _rsocketFlowable.Single.error(new Error('can not find service ' + service));
      }

      return handler.requestResponse(payload);
    } catch (error) {
      return _rsocketFlowable.Single.error(error);
    }
  };

  RequestHandlingRSocket.prototype.requestStream = function requestStream(payload) {
    try {
      if (payload.metadata == null) {
        return _rsocketFlowable.Flowable.error(new Error('metadata is empty'));
      }

      var service = (0, _rsocketRpcFrames.getService)(payload.metadata);
      var handler = this._registeredServices.get(service);

      if (handler == null) {
        return _rsocketFlowable.Flowable.error(new Error('can not find service ' + service));
      }

      return handler.requestStream(payload);
    } catch (error) {
      return _rsocketFlowable.Flowable.error(error);
    }
  };

  RequestHandlingRSocket.prototype.requestChannel = function requestChannel(payloads) {
    var _this = this;

    var payloadsProxy = new _QueuingFlowableProcessor2.default();
    var once = false;
    return new _rsocketFlowable.Flowable(function (subscriber) {
      payloads.subscribe({
        onNext: function onNext(payload) {
          if (!once) {
            once = true;
            try {
              if (payload.metadata === undefined || payload.metadata === null) {
                _rsocketFlowable.Flowable.error(new Error('metadata is empty')).subscribe(subscriber);
              } else {
                var service = (0, _rsocketRpcFrames.getService)(payload.metadata);
                var handler = _this._registeredServices.get(service);
                if (handler === undefined || handler === null) {
                  _rsocketFlowable.Flowable.error(new Error('can not find service ' + service)).subscribe(subscriber);
                } else {
                  handler.requestChannel(payloadsProxy).subscribe(subscriber);
                }
              }
            } catch (error) {
              //Message back to the caller
              subscriber.onError(error);
            }
          }
          payloadsProxy.onNext(payload);
        },
        onError: function onError(error) {
          payloadsProxy.onError(error);
        },
        onComplete: function onComplete() {
          payloadsProxy.onComplete();
        },
        onSubscribe: function onSubscribe(subscription) {
          payloadsProxy.onSubscribe(subscription);
        }
      });
    });
  };

  RequestHandlingRSocket.prototype.metadataPush = function metadataPush(payload) {
    return _rsocketFlowable.Single.error(new Error('metadataPush() is not implemented'));
  };

  return RequestHandlingRSocket;
}();

exports.default = RequestHandlingRSocket;